"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = require("react");

var _utils = require("./utils");

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var initialState = {
  loading: true,
  data: null,
  error: null
};

function reducer(state, action) {
  switch (action.type) {
    case "start":
      {
        return _objectSpread({}, state, {
          loading: true
        });
      }

    case "success":
      return {
        //...state,
        loading: false,
        data: action.payload,
        error: null
      };

    case "error":
      return _objectSpread({}, state, {
        loading: false,
        error: action.payload
      });

    default:
      throw new Error("useQuery/reducer(): Unknown action.type ".concat(action.type));
  }
} // options = { query, variables }


var useQuery = function useQuery(client, options) {
  var fnGetData = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : function (data) {
    return data;
  };

  var _useReducer = (0, _react.useReducer)(reducer, initialState),
      _useReducer2 = _slicedToArray(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  var memoOptions = (0, _react.useMemo)(function () {
    // if (process.env.NODE_ENV==='development') {
    //   console.log('useMemo memoRestOptions')
    // }      
    var variables = options.variables,
        restOptions = _objectWithoutProperties(options, ["variables"]);

    return _objectSpread({
      variables: variables || {}
    }, restOptions);
  }, // eslint-disable-next-line
  (0, _utils.options2arr)(options));
  var observableQuery = (0, _react.useMemo)(function () {
    var _observable;

    try {
      _observable = client.watchQuery(_objectSpread({}, memoOptions, {
        fetchPolicy: "network-only"
      }));
    } catch (errWatch) {
      dispatch({
        type: "error",
        payload: errWatch
      });
    }

    return _observable;
  }, [client, memoOptions]);
  (0, _react.useEffect)(function () {
    if (!observableQuery) {
      return;
    }

    var s = observableQuery.subscribe(function (resultNext) {
      var newData = resultNext.data;

      if (fnGetData) {
        newData = fnGetData(resultNext.data);
      }

      if (Array.isArray(newData)) {
        //newData = [ ...newData ]
        newData = newData.slice();
      } else if (!!newData && newData.constructor === Object) {
        //newData = { ...newData0 }
        newData = Object.assign({}, newData);
      }

      dispatch({
        type: 'success',
        payload: newData
      });
    }, function (err) {
      dispatch({
        type: 'error',
        payload: err
      });
    });
    return function () {
      return s.unsubscribe();
    };
  }, [observableQuery, fnGetData]);
  return {
    loading: state.loading,
    data: state.data,
    error: state.error
  };
};

var _default = useQuery;
exports.default = _default;